<?php

namespace General\Helpers;

use App\Services\ChannelEncode;
use General\Models\SendOrder\SendOrder;
use Illuminate\Log\Writer;
use Monolog\Logger;
use Vinkla\Hashids\Facades\Hashids;

class CommonHelper
{
    /**
     * 获取真实IP
     */
    public static function GetClientIp()
    {
        if (getenv('HTTP_X_FORWARDED_FOR')) {
            $ip = getenv('HTTP_X_FORWARDED_FOR');
        } else if (getenv("HTTP_CLIENT_IP") && strcasecmp(getenv("HTTP_CLIENT_IP"), "unknown"))
            $ip = getenv("HTTP_CLIENT_IP");
        else if (getenv("HTTP_X_FORWARD_FOR") && strcasecmp(getenv("HTTP_X_FORWARD_FOR"), "unknown"))
            $ip = getenv("HTTP_X_FORWARD_FOR");
        else if (getenv("REMOTE_ADDR") && strcasecmp(getenv("REMOTE_ADDR"), "unknown"))
            $ip = getenv("REMOTE_ADDR");
        else if (isset($_SERVER['REMOTE_ADDR']) && $_SERVER['REMOTE_ADDR'] && strcasecmp($_SERVER['REMOTE_ADDR'], "unknown"))
            $ip = $_SERVER['REMOTE_ADDR'];
        else
            $ip = "unknown";

        $ip = trim($ip);
        $new_ips = explode(',',$ip);
        $new_ip = isset($new_ips[0])?$new_ips[0]:$ip;
        $new_ip = trim($new_ip);
        \Log::info('GetClientIp_newip:'.$new_ip.' ip:'.$ip);
        return ($new_ip);
    }

    /**
     * 微信签名
     */
    public static function sign($params, $key)
    {
        $data = $params;
        //签名步骤一：按字典序排序参数
        ksort($data);
        $buff = "";
        foreach ($data as $k => $v) {
            if ($v != null && $k !== "sign" && $v !== "" && !is_array($v)) {
                $buff .= $k . "=" . $v . "&";
            }
        }
        $buff = trim($buff, "&");
        //签名步骤二：在string后加入KEY
        $string = $buff . "&key=" . $key;
        //签名步骤三：MD5加密
        $string = md5($string);
        //签名步骤四：所有字符转为大写
        $result = strtoupper($string);
        return $result;
    }

    /**
     * 自定义日志
     */
    public static function myLog($name, $filename = '')
    {
        if (!$filename) {
            $filename = $name;
        }
        $filename = $filename . '.log';
        $logger   = new Logger(storage_path('logs/' . $filename));
        $writer   = new Writer($logger);
        $writer->useDailyFiles(storage_path('logs/' . $filename));
        return $writer;
    }

    /**
     * 获取推广链接
     */
    public static function getPromotionUrl(SendOrder $send_order): string
    {
        if ($send_order->promotion_point == 2) {
            $domain = 'https://bsite';
        } else {
            $domain = 'https://site';
        }
        $service = new ChannelEncode;
        return $domain . $service->encodeChannelId($send_order->distribution_channel_id) . '.' . env('PROMOTION_DOMAIN') . '/yun/' . $send_order->id;
    }

    /**
     * bid 加密
     */
    public static function book_hash_encode($bid)
    {
        return Hashids::encode($bid);
    }

    /**
     * bid 解密
     */
    public static function book_hash_decode($encrypt_bid)
    {
        $decode_infos = Hashids::decode($encrypt_bid);
        return $decode_infos ? $decode_infos[0] : '';
    }

    /**
     * 
     */
    public static function getActivityUrl(int $channel_id, string $path): string
    {
        $service = new ChannelEncode;
        return env('PROTOCOL') . '://site' . $service->encodeChannelId($channel_id) . '.' . env('CUSTOM_HOST') . '.com' . $path;
    }

    public static function isInnerSites(int $distribution_channel_id)
    {
        $site = redisEnv('INNER_SITE');
        if (!$site) return false;
        return in_array($distribution_channel_id, explode(',', $site));
    }

    /**
     * 加密site id
     */
    public static function encodeDistributionChannelId($id)
    {
        $encrypt_pool = ['14' => 'xyvz5mexll52mzn4', '13' => 'laosiji', '4372' => 'qhyeyue', '365' => 'vciam5tg71', '384' => 'sdxisd', '5795' => 'gyp23fzu', '191' => 'jinsshxs'];
        if (isset($encrypt_pool[$id])) {
            return $encrypt_pool[$id];
        }
        /*$db_encrypt_info = \DB::table('distribution_channel_id_encrypt')
            ->where('distribution_channel_id',$id)
            ->where('is_enable',1)
            ->select('en_distribution_channel_id')
            ->first();

        if($db_encrypt_info && $db_encrypt_info->en_distribution_channel_id){
            return $db_encrypt_info->en_distribution_channel_id;
        }*/
        $hashids = new \Hashids\Hashids('', 16, 'abcdefghjklmnopqrstuvwxyz1234567890');
        return $hashids->encode($id);
    }
}
